@@ -346,7 +346,7 @@ static void on_entry_changed( GtkEntry* entry, GtkImage* img )
 
         gtk_icon_size_lookup(GTK_ICON_SIZE_DIALOG, &w, &h);
         fm_icon = fm_icon_from_name(name ? name : "application-x-executable");
-        pix = fm_pixbuf_from_icon_with_fallback(fm_icon, h, "application-x-executable");
+        pix = fm_pixbuf_from_icon_with_fallback(fm_icon, h, 1, "application-x-executable");
         g_object_unref(fm_icon);
         gtk_image_set_from_pixbuf(img, pix);
         g_object_unref(pix);
