@@ -393,6 +393,10 @@ static void create_search (MenuPlugin *m)
 
     /* set up the tree view */
     prend = gtk_cell_renderer_pixbuf_new ();
+    GValue val = G_VALUE_INIT;
+    g_value_init (&val, G_TYPE_INT);
+    g_value_set_int (&val, gtk_widget_get_scale_factor (m->img));
+    g_object_set_property (G_OBJECT (prend), "scale", &val);
     trend = gtk_cell_renderer_text_new ();
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (m->stv), -1, NULL, prend, "pixbuf", 0, NULL);
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (m->stv), -1, NULL, trend, "text", 1, NULL);
@@ -606,33 +610,35 @@ static GtkWidget *create_system_menu_item (MenuCacheItem *item, MenuPlugin *m)
 #ifdef LXPLUG
         FmIcon *fm_icon = fm_file_info_get_icon (fi);
         if (fm_icon == NULL) fm_icon = fm_icon_from_name ("application-x-executable");
-        icon = fm_pixbuf_from_icon_with_fallback (fm_icon, panel_get_safe_icon_size (m->panel), "application-x-executable");
+        icon = fm_pixbuf_from_icon_with_fallback (fm_icon, panel_get_safe_icon_size (m->panel), 1, "application-x-executable");
         gtk_image_set_from_pixbuf (GTK_IMAGE (img), icon);
 #else
         icon = NULL;
         const char *icon_name = menu_cache_item_get_icon (item);
+        int scale = gtk_widget_get_scale_factor (m->plugin);
         if (icon_name)
         {
             if (strstr (icon_name, "/"))
-                icon = gdk_pixbuf_new_from_file_at_size (icon_name, get_icon_size (), get_icon_size (), NULL);
+                icon = gdk_pixbuf_new_from_file_at_size (icon_name, get_icon_size () * scale, get_icon_size () * scale, NULL);
             else
             {
-                icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), icon_name,
-                    get_icon_size (), GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+                icon = gtk_icon_theme_load_icon_for_scale (gtk_icon_theme_get_default (), icon_name,
+                    get_icon_size (), scale, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
 
                 // fallback for packages using obsolete icon location
                 if (!icon)
                 {
                     char *fname = g_strdup_printf ("/usr/share/pixmaps/%s", icon_name);
-                    icon = gdk_pixbuf_new_from_file_at_size (fname, get_icon_size (), get_icon_size (), NULL);
+                    icon = gdk_pixbuf_new_from_file_at_size (fname, get_icon_size () * scale, get_icon_size () * scale, NULL);
                     g_free (fname);
                 }
             }
         }
         if (!icon)
-            icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "application-x-executable",
-                get_icon_size (), GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
-        if (icon) gtk_image_set_from_pixbuf (GTK_IMAGE (img), icon);
+            icon = gtk_icon_theme_load_icon_for_scale (gtk_icon_theme_get_default (), "application-x-executable",
+                get_icon_size (), scale, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+
+        if (icon) set_image_from_pixbuf (img, icon);
 #endif
         if (menu_cache_item_get_type (item) == MENU_CACHE_TYPE_APP)
         {
@@ -810,6 +816,7 @@ static GtkWidget *read_menu_item (MenuPlugin *m, char *disp_name, char *icon, vo
 {
     GtkWidget *item, *box, *label, *img;
     GdkPixbuf *pixbuf;
+    int scale;
 
     item = gtk_menu_item_new ();
     box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, MENU_ICON_SPACE);
@@ -819,11 +826,12 @@ static GtkWidget *read_menu_item (MenuPlugin *m, char *disp_name, char *icon, vo
     label = gtk_label_new (disp_name);
     g_signal_connect (G_OBJECT (item), "activate", (GCallback) handle_run_command, cmd);
 
+    scale = gtk_widget_get_scale_factor (m->plugin);
     img = gtk_image_new ();
-    pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), icon, wrap_icon_size (m), GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+    pixbuf = gtk_icon_theme_load_icon_for_scale (gtk_icon_theme_get_default (), icon, wrap_icon_size (m), scale, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
     if (pixbuf)
     {
-        gtk_image_set_from_pixbuf (GTK_IMAGE (img), pixbuf);
+        set_image_from_pixbuf (img, pixbuf);
         g_object_unref (pixbuf);
         gtk_container_add (GTK_CONTAINER (box), img);
     }
@@ -915,12 +923,7 @@ static void menu_button_clicked (GtkWidget *, MenuPlugin *m)
 /* Handler for system config changed message from panel */
 void menu_update_display (MenuPlugin *m)
 {
-    GdkPixbuf *pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "start-here", wrap_icon_size (m), GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
-    if (pixbuf)
-    {
-        gtk_image_set_from_pixbuf (GTK_IMAGE (m->img), pixbuf);
-        g_object_unref (pixbuf);
-    }
+    wrap_set_taskbar_icon (m, m->img, "start-here");
     if (m->img) gtk_widget_set_size_request (m->img, wrap_icon_size (m) + 2 * m->padding, -1);
 
     if (m->applist) gtk_list_store_clear (m->applist);
